@@ -10,7 +10,7 @@ For information on getting started, see README.md.
 from __future__ import absolute_import
 import sqlalchemy
 import sqlalchemy.orm.session
-from sqlalchemy import Index, Column, Integer, String, Float, ForeignKey, DateTime, Boolean, LargeBinary, create_engine, orm
+from sqlalchemy import Index, Column, Integer, String, Float, ForeignKey, DateTime, Boolean, create_engine, orm
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import relationship, backref, sessionmaker, clear_mappers, deferred
 from sqlalchemy import and_, or_
@@ -22,4 +22,4 @@ from .core import *
 from .query import *
 from . import properties
 
-__version__ = '1.3.1'
+__version__ = '1.5.0'
