@@ -2,11 +2,14 @@ from __future__ import absolute_import
 from sqlalchemy import Index, create_engine, inspect
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker, clear_mappers
+import os
 from .. import config
 from .. import log
 
 _verbose = False
 _internal_session=None
+_internal_session_args=None
+_internal_session_pid=None
 _engine=None
 Session=None
 
@@ -14,9 +17,14 @@ def get_default_session():
     """Get the default ORM session to be used when no other is specified.
 
     :rtype: sqlalchemy.orm.Session"""
-    global _internal_session
+    global _internal_session, _internal_session_pid, _internal_session_args
     if _internal_session is None:
         init_db()
+    elif _internal_session_pid!=os.getpid():
+        if _internal_session_args is None:
+            raise RuntimeError("The process has been forked, but no information is available to open a new connection to the database")
+        else:
+            init_db(*_internal_session_args)
     return _internal_session
 
 def set_default_session(session):
@@ -26,6 +34,8 @@ def set_default_session(session):
     will subsequently fail. """
     global _internal_session
     _internal_session = session
+    _internal_session_pid = os.getpid()
+    _internal_session_args = None
 
 def get_default_engine():
     """Get the default sqlalchemy engine to be used when no other is specified."""
@@ -46,15 +56,15 @@ from .creator import Creator
 from .simulation import Simulation, SimulationProperty
 from .tracking import TrackData, update_tracker_halos
 from .timestep import TimeStep
-from .halo import Halo
+from .halo import SimulationObjectBase
 from .halo_data import HaloProperty, HaloLink
 
 Index("halo_index", HaloProperty.__table__.c.halo_id)
 Index("name_halo_index", HaloProperty.__table__.c.name_id,
       HaloProperty.__table__.c.halo_id)
-Index("halo_timestep_index", Halo.__table__.c.timestep_id)
-Index("halo_creator_index", Halo.__table__.c.creator_id)
-Index("halo_finder_index", Halo.__table__.c.finder_id)
+Index("halo_timestep_index", SimulationObjectBase.__table__.c.timestep_id)
+Index("halo_creator_index", SimulationObjectBase.__table__.c.creator_id)
+Index("halo_finder_index", SimulationObjectBase.__table__.c.finder_id)
 Index("haloproperties_creator_index", HaloProperty.__table__.c.creator_id)
 Index("halolink_index", HaloLink.__table__.c.halo_from_id)
 Index("named_halolink_index", HaloLink.__table__.c.relation_id, HaloLink.__table__.c.halo_from_id)
@@ -126,15 +136,24 @@ def _check_and_upgrade_database(engine):
 
 
 def init_db(db_uri=None, timeout=30, verbose=None):
-    global _verbose, _internal_session, _engine, Session
+    global _verbose, _internal_session, _engine, Session, _internal_session_args, _internal_session_pid
     if db_uri is None:
         db_uri = config.db
 
     if '//' not in db_uri:
         db_uri = 'sqlite:///' + db_uri
 
-    _engine = create_engine(db_uri, echo=verbose or _verbose,
-                            isolation_level='READ UNCOMMITTED', connect_args={'timeout': timeout})
+    if db_uri.startswith("sqlite:///"):
+        connect_args = {"timeout": timeout}
+    else:
+        connect_args = {"connect_timeout": timeout}
+
+    _engine = create_engine(
+        db_uri,
+        echo=verbose or _verbose,
+        isolation_level='READ UNCOMMITTED',
+        connect_args=connect_args
+    )
 
     _check_and_upgrade_database(_engine)
 
@@ -142,7 +161,22 @@ def init_db(db_uri=None, timeout=30, verbose=None):
     _internal_session=Session()
     Base.metadata.create_all(_engine)
     creator.set_creator(None)
+    _internal_session_args = (db_uri, timeout, verbose)
+    _internal_session_pid = os.getpid() # stored so that we can detect when a fork happens
 
+def close_db():
+    global _engine
+    close_session()
+    if _engine is not None:
+        _engine.dispose()
+        _engine = None
+
+def close_session():
+    global Session, _internal_session
+    if Session is not None:
+        Session.close_all()
+        Session = None
+        _internal_session = None
 
 from .dictionary import _get_dict_cache_for_session, get_dict_id, get_or_create_dictionary_item
 
