@@ -13,7 +13,7 @@ import tangos as db
 from tangos import all_simulations
 from tangos import core, config
 from tangos.core import Base, get_or_create_dictionary_item, \
-    Creator, Simulation, TimeStep, Halo, HaloProperty, HaloLink
+    Creator, Simulation, TimeStep, SimulationObjectBase, HaloProperty, HaloLink
 from tangos.core.simulation import SimulationProperty
 from tangos.core.tracking import TrackData
 from tangos.query import get_simulation, get_halo
@@ -109,7 +109,7 @@ def _db_import_export(target_session, from_session, *sims):
 
             logger.info("Transferring objects for %s", ts_ext)
             for h_ext in ts_ext.objects:
-                h = Halo(ts, h_ext.halo_number, h_ext.finder_id, h_ext.finder_offset, h_ext.NDM,
+                h = SimulationObjectBase(ts, h_ext.halo_number, h_ext.finder_id, h_ext.finder_offset, h_ext.NDM,
                          h_ext.NStar, h_ext.NGas, h_ext.object_typecode)
                 h.external_id = h_ext.id
                 halos_this_ts.append(h)
@@ -296,12 +296,19 @@ def add_tracker(halo, size=None):
     core.get_default_session().commit()
 
 
-def grep_run(opts):
-    run = core.get_default_session().query(Creator).filter(
+def grep_run_id(opts):
+    matching_runs = core.get_default_session().query(Creator).filter(
         Creator.command_line.like("%" + opts.query + "%")).all()
-    for r in run:
+    for r in matching_runs:
         print(r.id, end=' ')
+    print('\n')
 
+def grep_run_info(opts):
+    matching_runs = core.get_default_session().query(Creator).filter(
+        Creator.command_line.like("%" + opts.query + "%")).all()
+    for r in matching_runs:
+        r.print_info()
+    return matching_runs
 
 def list_recent_runs(opts):
     n = opts.num
@@ -311,6 +318,17 @@ def list_recent_runs(opts):
         r.print_info()
 
 
+def _erase_run_content(run):
+    run.halolinks.delete()
+    run.halos.delete()
+    run.properties.delete()
+    run.timesteps.delete()
+    for s in run.simulations:
+        core.get_default_session().delete(s)
+    core.get_default_session().commit()
+    core.get_default_session().delete(run)
+    core.get_default_session().commit()
+
 def rem_run(id, confirm=True):
     run = core.get_default_session().query(Creator).filter_by(id=id).first()
 
@@ -324,22 +342,28 @@ def rem_run(id, confirm=True):
         print(""">>> type "yes" to continue""")
 
     if (not confirm) or input(":").lower() == "yes":
-        #for y in run.halolinks:
-        #    core.get_default_session().delete(y)
-        run.halolinks.delete()
-        run.halos.delete()
-        run.properties.delete()
-        run.timesteps.delete()
-        for s in run.simulations:
-            print(s)
-            core.get_default_session().delete(s)
-        core.get_default_session().commit()
-        core.get_default_session().delete(run)
-        core.get_default_session().commit()
+        _erase_run_content(run)
         print("OK")
     else:
         print("aborted")
 
+def grep_remove_runs(opts):
+    print('Will remove the following run(s): ')
+    matching_runs = grep_run_info(opts)
+
+    # Always ask for confirmation, as this action could be very destructive
+    print("You want to delete everything created by the above run(s)?")
+    print(""">>> type "yes" to continue""")
+
+    if input(":").lower() == "yes":
+        for r, run in enumerate(matching_runs):
+            print('Removing run ',r+1,' of ',len(matching_runs))
+            _erase_run_content(run)
+        print("Done")
+    else:
+        print("aborted")
+
+
 def rollback(options):
     if len(options.ids)>0:
         for run_id in options.ids:
@@ -390,7 +414,7 @@ def list_available_properties(options):
 
 def diff(options):
     from ..testing import db_diff
-    differ = db_diff.TangosDbDiff(options.uri1, options.uri2)
+    differ = db_diff.TangosDbDiff(options.uri1, options.uri2, ignore_keys=options.ignore_value_of)
     if options.simulation:
         differ.compare_simulation(options.simulation)
     elif options.timestep:
@@ -399,11 +423,14 @@ def diff(options):
         differ.compare_object(options.object)
     else:
         differ.compare()
-    return differ.failed
+    result = differ.failed
 
-    diff = db_diff.diff(options.uri1, options.uri2)
-    if diff:
+    if result:
+        logger.info("Differences found. Exiting with status 1.")
         sys.exit(1)
+    else:
+        logger.info("No differences found.")
+
 
 def main():
     print("""
@@ -415,7 +442,6 @@ def main():
 
     args = parser.parse_args()
     core.process_options(args)
-    core.init_db()
     args.func(args)
 
 
@@ -447,11 +473,24 @@ def get_argument_parser_and_subparsers():
     subparse_recentruns.add_argument("num", type=int,
                                      help="The number of runs to display, starting with the most recent")
 
-    subparse_greprun = subparse.add_parser("grep-runs",
-                                              help="List IDs of runs matching command line")
-    subparse_greprun.set_defaults(func=grep_run)
+    subparse_greprun = subparse.add_parser("grep-run-ids",
+                                          help="List IDs of runs matching command line input")
+    subparse_greprun.set_defaults(func=grep_run_id)
     subparse_greprun.add_argument("query", type=str,
                                      help="The sub-string to search for in the command line")
+                                     
+    subparse_grepruninfo = subparse.add_parser("grep-runs",
+                                          help="List details of runs matching command line input")
+    subparse_grepruninfo.set_defaults(func=grep_run_info)
+    subparse_grepruninfo.add_argument("query", type=str,
+                                     help="The sub-string to search for in the command line")
+                                                              
+    subparse_grepremove = subparse.add_parser("grep-remove",
+                                           help="Remove runs matching command line input")
+    subparse_grepremove.set_defaults(func=grep_remove_runs)
+    subparse_grepremove.add_argument("query", type=str,
+                                      help="The sub-string to search for in the command line")
+
 
     # The following subcommands currently do not work and is disabled:
     """
@@ -496,6 +535,7 @@ def get_argument_parser_and_subparsers():
     subparse_diff.add_argument("--simulation", type=str, help="Only compare the specified simulation", default=None)
     subparse_diff.add_argument("--timestep", type=str, help="Only compare the specified timestep", default=None)
     subparse_diff.add_argument("--object", type=str, help="Only compare the specified object", default=None)
+    subparse_diff.add_argument("--ignore-value-of", nargs="*", type=str, help="Ignore the value of the specified properties", default=None)
     subparse_diff.set_defaults(func=diff)
 
     subparse_list_available_properties = subparse.add_parser("list-possible-properties",
