@@ -8,6 +8,7 @@ from . import GenericTangosTool
 from ..util import proxy_object
 from ..util import timestep_object_cache
 import numpy as np
+import numbers
 
 class PropertyImporter(GenericTangosTool):
     tool_name = 'import-properties'
@@ -39,12 +40,17 @@ class PropertyImporter(GenericTangosTool):
             value = value.relative_to_timestep_cache(self._object_cache).resolve(self._session)
             if value is not None:
                 return core.halo_data.HaloLink(object, value, name)
-        elif np.issubdtype(type(value), np.float) or np.issubdtype(type(value), np.integer):
+        elif isinstance(value, numbers.Number):
             return core.halo_data.HaloProperty(object, name, value)
+        elif isinstance(value, np.ndarray):
+            if np.issubdtype(value.dtype, np.number):
+                return core.halo_data.HaloProperty(object, name, value)
+            else:
+                logger.warn("Ignoring stat file entry key='%s' value='%s' as the value is not a number or an array of numbers",
+                        name.text, value)
         elif value is not None:
-            logger.warn("Ignoring stat file entry key='%s' value='%s' as the value is not a number",
+            logger.warn("Ignoring stat file entry key='%s' value='%s' as the value is not a number or an array of numbers",
                         name.text, value)
-
         return None
 
     def _create_properties(self, name, object, values):
