@@ -44,7 +44,7 @@ def format_data(data, request=None, relative_to=None, max_array_length=3):
         return format_array(data, max_array_length)
     elif np.issubdtype(type(data), np.number):
         return format_number(data)
-    elif isinstance(data, core.Halo):
+    elif isinstance(data, core.SimulationObjectBase):
         return format_halo(data, request, relative_to)
     else:
         return escape(repr(data))
