@@ -6,6 +6,7 @@ import numpy as np
 from .halo_data import format_number, _relative_description
 import sqlalchemy, sqlalchemy.orm
 from six.moves import zip
+import urllib.parse
 from . import halo_from_request
 
 class TimestepInfo(object):
@@ -68,7 +69,10 @@ def format_property_data(property):
         """
         return "Array"
     else:
-        return format_number(property.data)
+        try:
+            return format_number(property.data)
+        except:
+            return "(Exception while formatting data)"
 
 class SimulationInfo(object):
     def __init__(self, sim, request):
@@ -96,7 +100,7 @@ def halo_links(halo, request):
     links = []
     links_query = request.dbsession.query(core.HaloLink).filter_by(halo_from_id=halo.id).\
             order_by(core.HaloLink.weight.desc()).\
-            options(sqlalchemy.orm.joinedload(core.HaloLink.halo_to).joinedload(core.Halo.timestep).joinedload(core.TimeStep.simulation))
+            options(sqlalchemy.orm.joinedload(core.HaloLink.halo_to).joinedload(core.SimulationObjectBase.timestep).joinedload(core.TimeStep.simulation))
 
     for lk in links_query.all():
         links.append(HaloLinkInfo(lk, request))
@@ -129,4 +133,4 @@ def halo_view(request):
                                             halonumber=request.matchdict['halonumber']),
             'timestep_url': request.route_url('timestep_view',simid=request.matchdict['simid'],
                                             timestepid=request.matchdict['timestepid']),
-            'cascade_url': "/%s/%s/%s/"%(sim.escaped_basename,ts.escaped_extension,halo.basename)}
\ No newline at end of file
+            'cascade_url': urllib.parse.quote("/%s/%s/%s/"%(sim.escaped_basename,ts.escaped_extension,halo.basename))}
\ No newline at end of file
